@@ -4,7 +4,6 @@ import os
 
 from octue.cloud.pub_sub.service import Service
 from octue.configuration import load_service_and_app_configuration
-from octue.exceptions import MissingServiceID
 from octue.resources.service_backends import GCPPubSubBackend
 from octue.runner import Runner
 from octue.utils.objects import get_nested_attribute
@@ -17,31 +16,25 @@ DEFAULT_SERVICE_CONFIGURATION_PATH = "octue.yaml"
 
 
 def answer_question(question, project_name):
-    """Answer a question from a service by running the deployed app with the deployment configuration. Either the
-    `deployment_configuration_path` should be specified, or the `deployment_configuration`.
+    """Answer a question sent to an app deployed in Google Cloud.
 
     :param dict|tuple|apache_beam.io.gcp.pubsub.PubsubMessage question:
     :param str project_name:
     :return None:
     """
-    service_id = os.environ.get("SERVICE_ID")
+    service_configuration, app_configuration = load_service_and_app_configuration(DEFAULT_SERVICE_CONFIGURATION_PATH)
+    service_id = os.environ.get("SERVICE_ID") or service_configuration.service_id
 
-    if not service_id:
-        raise MissingServiceID(
-            "The ID for the deployed service is missing or empty - ensure SERVICE_ID is available as an environment "
-            "variable."
-        )
+    service = Service(
+        name=service_id,
+        service_id=service_id,
+        backend=GCPPubSubBackend(project_name=project_name),
+    )
 
     question_uuid = get_nested_attribute(question, "attributes.question_uuid")
-
-    service = Service(service_id=service_id, backend=GCPPubSubBackend(project_name=project_name))
-
     answer_topic = service.instantiate_answer_topic(question_uuid)
 
     try:
-        service_configuration, app_configuration = load_service_and_app_configuration(
-            DEFAULT_SERVICE_CONFIGURATION_PATH
-        )
         service.name = service_configuration.name
 
         runner = Runner(
@@ -52,6 +45,7 @@ def answer_question(question, project_name):
             children=app_configuration.children,
             output_location=app_configuration.output_location,
             project_name=project_name,
+            service_id=service_id,
         )
 
         service.run_function = functools.partial(runner.run)
@@ -61,5 +55,5 @@ def answer_question(question, project_name):
 
     # Forward any errors in the deployment configuration (errors in the analysis are already forwarded by the service).
     except BaseException as error:  # noqa
-        service.send_exception_to_asker(topic=answer_topic)
+        service.send_exception(topic=answer_topic)
         logger.exception(error)
