@@ -9,9 +9,6 @@ VALUES_FILENAME = "values.json"
 MANIFEST_FILENAME = "manifest.json"
 CHILDREN_FILENAME = "children.json"
 
-# Default JSON filenames for storing metadata about non-file objects in the cloud.
-DATASET_METADATA_FILENAME = "dataset_metadata.json"
-
 STRAND_FILENAME_MAP = {
     "configuration_values": VALUES_FILENAME,
     "configuration_manifest": MANIFEST_FILENAME,
@@ -21,7 +18,6 @@ STRAND_FILENAME_MAP = {
     "output_manifest": MANIFEST_FILENAME,
 }
 
-
 # TODO this should probably be defined in twined
 OUTPUT_STRANDS = ("output_values", "output_manifest")
 
