@@ -6,8 +6,6 @@ import logging
 import os
 import shutil
 import tempfile
-import warnings
-from urllib.parse import urlparse
 
 import google.api_core.exceptions
 import pkg_resources
@@ -26,12 +24,10 @@ except ModuleNotFoundError:
 from octue.cloud import storage
 from octue.cloud.storage import GoogleCloudStorageClient
 from octue.exceptions import CloudLocationNotSpecified, ReadOnlyResource
-from octue.migrations.cloud_storage import translate_bucket_name_and_path_in_bucket_to_cloud_path
-from octue.mixins import Filterable, Hashable, Identifiable, Labelable, Metadata, Serialisable, Taggable
+from octue.mixins import CloudPathable, Filterable, Hashable, Identifiable, Labelable, Metadata, Serialisable, Taggable
 from octue.mixins.hashable import EMPTY_STRING_HASH_VALUE
 from octue.utils.decoders import OctueJSONDecoder
-from octue.utils.encoders import OctueJSONEncoder
-from octue.utils.metadata import METADATA_FILENAME, load_local_metadata_file
+from octue.utils.metadata import METADATA_FILENAME, load_local_metadata_file, overwrite_local_metadata_file
 
 
 logger = logging.getLogger(__name__)
@@ -40,7 +36,7 @@ logger = logging.getLogger(__name__)
 OCTUE_METADATA_NAMESPACE = "octue"
 
 
-class Datafile(Labelable, Taggable, Serialisable, Identifiable, Hashable, Filterable, Metadata):
+class Datafile(Labelable, Taggable, Serialisable, Identifiable, Hashable, Filterable, Metadata, CloudPathable):
     """A representation of a data file on the Octue system. Metadata for the file is obtained from its corresponding
     cloud object or a local `.octue` metadata file, if present. If no stored metadata is available, it can be set during
     or after instantiation.
@@ -49,15 +45,17 @@ class Datafile(Labelable, Taggable, Serialisable, Identifiable, Hashable, Filter
     :param str|None local_path: If a cloud path is given as the `path` parameter, this is the path to an existing local file that is known to be in sync with the cloud object
     :param str|None cloud_path: If a local path is given for the `path` parameter, this is a cloud path to keep in sync with the local file
     :param datetime.datetime|int|float|None timestamp: A posix timestamp associated with the file, in seconds since epoch, typically when it was created but could relate to a relevant time point for the data
-    :param str id: The Universally Unique ID of this file (checked to be valid if not None, generated if None)
-    :param dict|octue.resources.tag.TagDict|None tags: key-value pairs with string keys conforming to the Octue tag format (see TagDict)
-    :param iter(str)|octue.resources.label.LabelSet|None labels: Space-separated string of labels relevant to this file
     :param str mode: if using as a context manager, open the datafile for reading/editing in this mode (the mode options are the same as for the builtin `open` function)
     :param bool update_metadata: if using as a context manager and this is `True`, update the stored metadata of the datafile when the context is exited
     :param bool hypothetical: if `True`, ignore any metadata stored for this datafile locally or in the cloud and use whatever is given at instantiation
+    :param str id: The Universally Unique ID of this file (checked to be valid if not None, generated if None)
+    :param dict|octue.resources.tag.TagDict|None tags: key-value pairs with string keys conforming to the Octue tag format (see `TagDict`)
+    :param iter(str)|octue.resources.label.LabelSet|None labels: Space-separated string of labels relevant to this file
     :return None:
     """
 
+    _CLOUD_PATH_ATTRIBUTE_NAME = "cloud_path"
+
     _METADATA_ATTRIBUTES = ("id", "timestamp", "tags", "labels")
 
     _SERIALISE_FIELDS = (
@@ -74,26 +72,14 @@ class Datafile(Labelable, Taggable, Serialisable, Identifiable, Hashable, Filter
         local_path=None,
         cloud_path=None,
         timestamp=None,
-        id=None,
-        tags=None,
-        labels=None,
         mode="r",
         update_metadata=True,
         hypothetical=False,
+        id=None,
+        tags=None,
+        labels=None,
         **kwargs,
     ):
-        if "update_cloud_metadata" in kwargs:
-            warnings.warn(
-                message=(
-                    "The `Datafile` instantiation parameter `update_cloud_metadata` has been deprecated and renamed to "
-                    "`update_metadata`. The old name will become unavailable soon, so please update to the new one. It "
-                    "has been translated for now."
-                ),
-                category=DeprecationWarning,
-            )
-
-            update_metadata = kwargs["update_cloud_metadata"]
-
         super().__init__(
             id=id,
             name=kwargs.pop("name", None),
@@ -193,38 +179,7 @@ class Datafile(Labelable, Taggable, Serialisable, Identifiable, Hashable, Filter
             self._cloud_path = None
 
         else:
-            self.to_cloud(cloud_path=path)
-
-    @property
-    def cloud_protocol(self):
-        """Get the cloud protocol of the datafile if it exists in the cloud (e.g. "gs" for a cloud path of
-        "gs://my-bucket/my-file.txt").
-
-        :return str|None:
-        """
-        if not self.exists_in_cloud:
-            return None
-        return urlparse(self.cloud_path).scheme
-
-    @property
-    def bucket_name(self):
-        """Get the name of the bucket the datafile exists in if it exists in the cloud.
-
-        :return str|None:
-        """
-        if self.cloud_path:
-            return storage.path.split_bucket_name_from_cloud_path(self.cloud_path)[0]
-        return None
-
-    @property
-    def path_in_bucket(self):
-        """Get the path of the datafile in its bucket if it exists in the cloud.
-
-        :return str|None:
-        """
-        if self.cloud_path:
-            return storage.path.split_bucket_name_from_cloud_path(self.cloud_path)[1]
-        return None
+            self.upload(cloud_path=path)
 
     @property
     def cloud_hash_value(self):
@@ -303,14 +258,6 @@ class Datafile(Labelable, Taggable, Serialisable, Identifiable, Hashable, Filter
         except FileNotFoundError:
             return None
 
-    @property
-    def exists_in_cloud(self):
-        """Return `True` if the file exists in the cloud.
-
-        :return bool:
-        """
-        return self.cloud_path is not None
-
     @property
     def exists_locally(self):
         """Return `True` if the file exists locally.
@@ -410,16 +357,13 @@ class Datafile(Labelable, Taggable, Serialisable, Identifiable, Hashable, Filter
             raise TypeError(f"An object of type {type(self)} cannot be compared with {type(other)}.")
         return self.name > other.name
 
-    def to_cloud(self, cloud_path=None, bucket_name=None, path_in_bucket=None, update_cloud_metadata=True):
+    def upload(self, cloud_path=None, update_cloud_metadata=True):
         """Upload a datafile to Google Cloud Storage.
 
         :param str|None cloud_path: full path to cloud storage location to store datafile at (e.g. `gs://bucket_name/path/to/file.csv`)
         :param bool update_cloud_metadata: if `True`, update the metadata of the datafile in the cloud at upload time
         :return str: gs:// path for datafile
         """
-        if bucket_name:
-            cloud_path = translate_bucket_name_and_path_in_bucket_to_cloud_path(bucket_name, path_in_bucket)
-
         cloud_path = self._get_cloud_location(cloud_path)
 
         self._get_cloud_metadata()
@@ -530,9 +474,7 @@ class Datafile(Labelable, Taggable, Serialisable, Identifiable, Hashable, Filter
             existing_metadata_records["datafiles"] = {}
 
         existing_metadata_records["datafiles"][self.name] = self.metadata(use_octue_namespace=False)
-
-        with open(self._local_metadata_path, "w") as f:
-            json.dump(existing_metadata_records, f, cls=OctueJSONEncoder)
+        overwrite_local_metadata_file(data=existing_metadata_records, path=self._local_metadata_path)
 
     def generate_signed_url(self, expiration=datetime.timedelta(days=7)):
         """Generate a signed URL for the datafile.
@@ -668,33 +610,6 @@ class Datafile(Labelable, Taggable, Serialisable, Identifiable, Hashable, Filter
         else:
             return self.cloud_hash_value or EMPTY_STRING_HASH_VALUE
 
-    def _get_cloud_location(self, cloud_path=None):
-        """Get the cloud location details for the bucket, allowing the keyword arguments to override any stored values.
-        Once the cloud location details have been determined, update the stored cloud location details.
-
-        :param str|None cloud_path:
-        :raise octue.exceptions.CloudLocationNotSpecified: if an exact cloud location isn't provided and isn't available implicitly (i.e. the Datafile wasn't loaded from the cloud previously)
-        :return (str, str): project name and cloud path
-        """
-        cloud_path = cloud_path or self.cloud_path
-
-        if not cloud_path:
-            self._raise_cloud_location_error()
-
-        self._cloud_path = cloud_path
-        return cloud_path
-
-    def _raise_cloud_location_error(self):
-        """Raise an error indicating that the cloud location of the datafile has not yet been specified.
-
-        :raise CloudLocationNotSpecified:
-        :return None:
-        """
-        raise CloudLocationNotSpecified(
-            f"{self!r} wasn't previously loaded from the cloud so doesn't have an implicit cloud location - please "
-            f"specify its exact location (its project name and cloud path)."
-        )
-
 
 class _DatafileContextManager:
     """A context manager for opening datafiles for reading and writing locally or from the cloud. Its usage is analogous
@@ -768,7 +683,7 @@ class _DatafileContextManager:
         if any(character in self.mode for character in self.MODIFICATION_MODES):
 
             if self.datafile.exists_in_cloud:
-                self.datafile.to_cloud(update_cloud_metadata=self._update_metadata)
+                self.datafile.upload(update_cloud_metadata=self._update_metadata)
 
             elif self._update_metadata:
                 self.datafile.update_local_metadata()
